@@ -16,7 +16,7 @@ import spatialpandas
 
 class MapSource(object):
 
-    def __init__(self,
+    def __init__(self,  # noqa: C901
                  name=None,
                  description=None,
                  filepath=None,
@@ -52,7 +52,7 @@ class MapSource(object):
                  preload=False):
 
         if fields is None and isinstance(data, (gpd.GeoDataFrame)):
-            fields = ['geometry']
+            fields = [geometry_field]
             if zfield:
                 fields.append(zfield)
 
@@ -72,13 +72,15 @@ class MapSource(object):
             raise ValueError('You must include a zfield for min/max scan calculation')
 
         if legend is not None and geometry_type == 'raster':
-            cmap = {}
-            for leg in legend:
-                val = leg['value']
-                cor = leg['color']
-                if isinstance(val, (list, tuple)):
-                    val = tuple(val)
-                cmap[val] = cor
+
+            if legend[0].get('value') is not None:
+                cmap = {}
+                for leg in legend:
+                    cor = leg['color']
+                    val = leg['value']
+                    if isinstance(val, (list, tuple)):
+                        val = tuple(val)
+                    cmap[val] = cor
 
         val = 20037508.3427892
         if default_extent is None:
@@ -629,15 +631,15 @@ def parse_sources(source_objs, config_path=None, contains=None):
     }
 
     for source in source_objs:
+        # create sources
+        source_obj = MapSource.from_obj(source)
+
         for service_type in source['service_types']:
             source['config_path'] = config_path
 
             if contains and contains not in source.get('key'):
                 continue
 
-            # create sources
-            source_obj = MapSource.from_obj(source)
-
             # create services
             ServiceKlass = service_classes[service_type]
 
